/// @addtogroup unifexp
/// @{

/////////////////////////////////////////////////////////////////////////////
///
/// @file intvcapd.h
///
/// This file contains some definitions necessary to use intervals
/// from the CAPD library.
/// Please, note that in the most of the code, BOOST intervals are used.
/// However, one may prefer to use the CAPD intervals occassionally,
/// because of the additional functions (sin, cos, log, exp) defined there.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2013 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started on February 3, 2013. Last revision: October 4, 2013.

#ifndef _intvcapd_h_
#define _intvcapd_h_

#include "capd/intervals/lib.h"
#include "capd/rounding/DoubleRounding.h"


namespace unifexp {

// --------------------------------------------------
// ------------- intervals and rounding -------------
// --------------------------------------------------

/// The type of an interval (from the CAPD library 2.9/3.0 beta).
typedef capd::DInterval IntervalType;

/// Resets the rounding switches of the processor
/// and sets rounding mode to the default one (to the nearest).
/// It is necessary in some cases, e.g., if the CAPD leaves
/// incorrect rounding mode after doing some interval operations.
inline void resetRounding ()
{
	capd::rounding::DoubleRounding::roundNearest ();
	return;
} /* resetRounding */

inline IntervalType intervalExp (const IntervalType &x)
{
	if (x. rightBound () > 700)
		throw "Exp: Argument too large for an interval on doubles.";
	return exp (x);
} /* intervalExp */

/// A substitute of the interval "power" function.
/// This seems to be necessary because of some bug
/// which makes it compute power (IntervalType (5, 5), 0.89) == 1.
inline IntervalType intervalPower (const IntervalType &x,
	const IntervalType &y)
{
	if (x. leftBound () <= 0)
		throw "Power: Non-positive base supplied.";
	return intervalExp (y * log (x));
} /* intervalPower */

//using capd::intervals::power;

} // namespace unifexp

#endif // _intvcapd_h_

/// @}

